@@ -8,6 +8,7 @@
 import asyncio
 import collections
 import collections.abc
+import itertools
 import struct
 import time
 import warnings
@@ -37,12 +38,13 @@ class Connection(metaclass=ConnectionMeta):
     Connections are created by calling :func:`~asyncpg.connection.connect`.
     """
 
-    __slots__ = ('_protocol', '_transport', '_loop', '_types_stmt',
-                 '_type_by_name_stmt', '_top_xact', '_uid', '_aborted',
-                 '_stmt_cache', '_stmts_to_close', '_listeners',
-                 '_server_version', '_server_caps', '_intro_query',
-                 '_reset_query', '_proxy', '_stmt_exclusive_section',
-                 '_config', '_params', '_addr', '_log_listeners')
+    __slots__ = ('_protocol', '_transport', '_loop',
+                 '_top_xact', '_uid', '_aborted',
+                 '_pool_release_ctr', '_stmt_cache', '_stmts_to_close',
+                 '_listeners', '_server_version', '_server_caps',
+                 '_intro_query', '_reset_query', '_proxy',
+                 '_stmt_exclusive_section', '_config', '_params', '_addr',
+                 '_log_listeners')
 
     def __init__(self, protocol, transport, loop,
                  addr: (str, int) or str,
@@ -51,11 +53,13 @@ class Connection(metaclass=ConnectionMeta):
         self._protocol = protocol
         self._transport = transport
         self._loop = loop
-        self._types_stmt = None
-        self._type_by_name_stmt = None
         self._top_xact = None
         self._uid = 0
         self._aborted = False
+        # Incremented very time the connection is released back to a pool.
+        # Used to catch invalid references to connection-related resources
+        # post-release (e.g. explicit prepared statements).
+        self._pool_release_ctr = 0
 
         self._addr = addr
         self._config = config
@@ -80,10 +84,7 @@ class Connection(metaclass=ConnectionMeta):
         self._server_caps = _detect_server_capabilities(
             self._server_version, settings)
 
-        if self._server_version < (9, 2):
-            self._intro_query = introspection.INTRO_LOOKUP_TYPES_91
-        else:
-            self._intro_query = introspection.INTRO_LOOKUP_TYPES
+        self._intro_query = introspection.INTRO_LOOKUP_TYPES
 
         self._reset_query = None
         self._proxy = None
@@ -283,14 +284,17 @@ class Connection(metaclass=ConnectionMeta):
             stmt_name = ''
 
         statement = await self._protocol.prepare(stmt_name, query, timeout)
-
         ready = statement._init_types()
         if ready is not True:
-            if self._types_stmt is None:
-                self._types_stmt = await self.prepare(self._intro_query)
-
-            types = await self._types_stmt.fetch(list(ready))
+            types, intro_stmt = await self.__execute(
+                self._intro_query, (list(ready),), 0, timeout)
             self._protocol.get_settings().register_data_types(types)
+            if not intro_stmt.name and not statement.name:
+                # The introspection query has used an anonymous statement,
+                # which has blown away the anonymous statement we've prepared
+                # for the query, so we need to re-prepare it.
+                statement = await self._protocol.prepare(
+                    stmt_name, query, timeout)
 
         if use_cache:
             self._stmt_cache.put(query, statement)
@@ -352,7 +356,8 @@ class Connection(metaclass=ConnectionMeta):
                             ``command_timeout`` argument to the ``Connection``
                             instance constructor.
 
-        :return: The value of the specified column of the first record.
+        :return: The value of the specified column of the first record, or
+                 None if no records were returned by the query.
         """
         self._check_open()
         data = await self._execute(query, args, 1, timeout)
@@ -367,7 +372,8 @@ class Connection(metaclass=ConnectionMeta):
         :param args: Query arguments
         :param float timeout: Optional timeout value in seconds.
 
-        :return: The first row as a :class:`Record` instance.
+        :return: The first row as a :class:`Record` instance, or None if
+                 no records were returned by the query.
         """
         self._check_open()
         data = await self._execute(query, args, 1, timeout)
@@ -827,9 +833,6 @@ class Connection(metaclass=ConnectionMeta):
             Callable accepting a single argument encoded according to *format*
             and returning a decoded Python object.
 
-        :param binary:
-            **Deprecated**.  Use *format* instead.
-
         Example:
 
         .. code-block:: pycon
@@ -870,23 +873,14 @@ class Connection(metaclass=ConnectionMeta):
             The ``binary`` keyword argument is deprecated in favor of
             ``format``.
 
+        .. versionchanged:: 0.13.0
+            The ``binary`` keyword argument was removed in favor of
+            ``format``.
         """
         self._check_open()
 
-        if binary is not None:
-            format = 'binary' if binary else 'text'
-            warnings.warn(
-                "The `binary` keyword argument to "
-                "set_type_codec() is deprecated and will be removed in "
-                "asyncpg 0.13.0.  Use the `format` keyword argument instead.",
-                DeprecationWarning, stacklevel=2)
-
-        if self._type_by_name_stmt is None:
-            self._type_by_name_stmt = await self.prepare(
-                introspection.TYPE_BY_NAME)
-
-        typeinfo = await self._type_by_name_stmt.fetchrow(
-            typename, schema)
+        typeinfo = await self.fetchrow(
+            introspection.TYPE_BY_NAME, typename, schema)
         if not typeinfo:
             raise ValueError('unknown type: {}.{}'.format(schema, typename))
 
@@ -916,12 +910,8 @@ class Connection(metaclass=ConnectionMeta):
         .. versionadded:: 0.12.0
         """
 
-        if self._type_by_name_stmt is None:
-            self._type_by_name_stmt = await self.prepare(
-                introspection.TYPE_BY_NAME)
-
-        typeinfo = await self._type_by_name_stmt.fetchrow(
-            typename, schema)
+        typeinfo = await self.fetchrow(
+            introspection.TYPE_BY_NAME, typename, schema)
         if not typeinfo:
             raise ValueError('unknown type: {}.{}'.format(schema, typename))
 
@@ -944,12 +934,8 @@ class Connection(metaclass=ConnectionMeta):
         """
         self._check_open()
 
-        if self._type_by_name_stmt is None:
-            self._type_by_name_stmt = await self.prepare(
-                introspection.TYPE_BY_NAME)
-
-        typeinfo = await self._type_by_name_stmt.fetchrow(
-            typename, schema)
+        typeinfo = await self.fetchrow(
+            introspection.TYPE_BY_NAME, typename, schema)
         if not typeinfo:
             raise ValueError('unknown type: {}.{}'.format(schema, typename))
 
@@ -1168,6 +1154,29 @@ class Connection(metaclass=ConnectionMeta):
 
         self._proxy = proxy
 
+    def _check_listeners(self, listeners, listener_type):
+        if listeners:
+            count = len(listeners)
+
+            w = exceptions.InterfaceWarning(
+                '{conn!r} is being released to the pool but has {c} active '
+                '{type} listener{s}'.format(
+                    conn=self, c=count, type=listener_type,
+                    s='s' if count > 1 else ''))
+
+            warnings.warn(w)
+
+    def _on_release(self, stacklevel=1):
+        # Invalidate external references to the connection.
+        self._pool_release_ctr += 1
+        # Called when the connection is about to be released to the pool.
+        # Let's check that the user has not left any listeners on it.
+        self._check_listeners(
+            list(itertools.chain.from_iterable(self._listeners.values())),
+            'notification')
+        self._check_listeners(
+            self._log_listeners, 'log')
+
     def _drop_local_statement_cache(self):
         self._stmt_cache.clear()
 
@@ -1181,18 +1190,25 @@ class Connection(metaclass=ConnectionMeta):
             self._drop_local_statement_cache()
 
     async def _execute(self, query, args, limit, timeout, return_status=False):
+        with self._stmt_exclusive_section:
+            result, _ = await self.__execute(
+                query, args, limit, timeout, return_status=return_status)
+        return result
+
+    async def __execute(self, query, args, limit, timeout,
+                        return_status=False):
         executor = lambda stmt, timeout: self._protocol.bind_execute(
             stmt, args, '', limit, return_status, timeout)
         timeout = self._protocol._get_timeout(timeout)
-        with self._stmt_exclusive_section:
-            return await self._do_execute(query, executor, timeout)
+        return await self._do_execute(query, executor, timeout)
 
     async def _executemany(self, query, args, timeout):
         executor = lambda stmt, timeout: self._protocol.bind_execute_many(
             stmt, args, '', timeout)
         timeout = self._protocol._get_timeout(timeout)
         with self._stmt_exclusive_section:
-            return await self._do_execute(query, executor, timeout)
+            result, _ = await self._do_execute(query, executor, timeout)
+        return result
 
     async def _do_execute(self, query, executor, timeout, retry=True):
         if timeout is None:
@@ -1214,7 +1230,7 @@ class Connection(metaclass=ConnectionMeta):
                     after = time.monotonic()
                     timeout -= after - before
 
-        except exceptions.InvalidCachedStatementError as e:
+        except exceptions.InvalidCachedStatementError:
             # PostgreSQL will raise an exception when it detects
             # that the result type of the query has changed from
             # when the statement was prepared.  This may happen,
@@ -1241,10 +1257,10 @@ class Connection(metaclass=ConnectionMeta):
             if self._protocol.is_in_transaction() or not retry:
                 raise
             else:
-                result = await self._do_execute(
+                return await self._do_execute(
                     query, executor, timeout, retry=False)
 
-        return result
+        return result, stmt
 
 
 async def connect(dsn=None, *,
