@@ -143,7 +143,8 @@ class PoolConnectionHolder:
         self._con = con
 
     async def acquire(self) -> PoolConnectionProxy:
-        if self._con is None:
+        if self._con is None or self._con.is_closed():
+            self._con = None
             await self.connect()
 
         self._maybe_cancel_inactive_callback()
@@ -175,6 +176,8 @@ class PoolConnectionHolder:
         assert self._in_use
         self._in_use = False
 
+        self._con._on_release()
+
         if self._con.is_closed():
             self._con = None
 
@@ -584,6 +587,15 @@ def create_pool(dsn=None, *,
         finally:
             await pool.release(con)
 
+    .. warning::
+        Prepared statements and cursors returned by
+        :meth:`Connection.prepare() <connection.Connection.prepare>` and
+        :meth:`Connection.cursor() <connection.Connection.cursor>` become
+        invalid once the connection is released.  Likewise, all notification
+        and log listeners are removed, and ``asyncpg`` will issue a warning
+        if there are any listener callbacks registered on a connection that
+        is being released to the pool.
+
     :param str dsn:
         Connection arguments specified using as a single string in
         the following format:
@@ -630,6 +642,19 @@ def create_pool(dsn=None, *,
     .. versionchanged:: 0.10.0
        An :exc:`~asyncpg.exceptions.InterfaceError` will be raised on any
        attempted operation on a released connection.
+
+    .. versionchanged:: 0.13.0
+       An :exc:`~asyncpg.exceptions.InterfaceError` will be raised on any
+       attempted operation on a prepared statement or a cursor created
+       on a connection that has been released to the pool.
+
+    .. versionchanged:: 0.13.0
+       An :exc:`~asyncpg.exceptions.InterfaceWarning` will be produced
+       if there are any active listeners (added via
+       :meth:`Connection.add_listener() <connection.Connection.add_listener>`
+       or :meth:`Connection.add_log_listener()
+       <connection.Connection.add_log_listener>`) present on the connection
+       at the moment of its release to the pool.
     """
     if not issubclass(connection_class, connection.Connection):
         raise TypeError(
