@@ -11,7 +11,6 @@ import ipaddress
 import math
 import random
 import struct
-import unittest
 import uuid
 
 import asyncpg
@@ -214,7 +213,7 @@ type_samples = [
     ('json', 'json', [
         '[1, 2, 3, 4]',
         '{"a": [1, 2], "b": 0}'
-    ], (9, 2)),
+    ]),
     ('jsonb', 'jsonb', [
         '[1, 2, 3, 4]',
         '{"a": [1, 2], "b": 0}'
@@ -303,7 +302,32 @@ type_samples = [
             output=ipaddress.IPv4Network('127.0.0.0/8')),
         dict(
             input='127.0.0.1/32',
-            output=ipaddress.IPv4Network('127.0.0.1/32')),
+            output=ipaddress.IPv4Address('127.0.0.1')),
+        # Postgres appends /32 when casting to text explicitly, but
+        # *not* in inet_out.
+        dict(
+            input='10.11.12.13',
+            textoutput='10.11.12.13/32'
+        ),
+        dict(
+            input=ipaddress.IPv4Address('10.11.12.13'),
+            textoutput='10.11.12.13/32'
+        ),
+        dict(
+            input=ipaddress.IPv4Network('10.11.12.13'),
+            textoutput='10.11.12.13/32'
+        ),
+        dict(
+            textinput='10.11.12.13',
+            output=ipaddress.IPv4Address('10.11.12.13'),
+        ),
+        dict(
+            # Non-zero address bits after the network prefix are permitted
+            # by postgres, but are invalid in Python
+            # (and zeroed out by supernet()).
+            textinput='10.11.12.13/0',
+            output=ipaddress.IPv4Network('0.0.0.0/0'),
+        ),
     ]),
     ('macaddr', 'macaddr', [
         '00:00:00:00:00:00',
@@ -370,20 +394,33 @@ class TestCodecs(tb.ConnectedTestCase):
                 "SELECT $1::" + typname
             )
 
-            textst = await self.con.prepare(
+            text_in = await self.con.prepare(
                 "SELECT $1::text::" + typname
             )
 
+            text_out = await self.con.prepare(
+                "SELECT $1::" + typname + "::text"
+            )
+
             for sample in sample_data:
                 with self.subTest(sample=sample, typname=typname):
                     stmt = st
                     if isinstance(sample, dict):
                         if 'textinput' in sample:
                             inputval = sample['textinput']
-                            stmt = textst
+                            stmt = text_in
                         else:
                             inputval = sample['input']
-                        outputval = sample['output']
+
+                        if 'textoutput' in sample:
+                            outputval = sample['textoutput']
+                            if stmt is text_in:
+                                raise ValueError(
+                                    'cannot test "textin" and'
+                                    ' "textout" simultaneously')
+                            stmt = text_out
+                        else:
+                            outputval = sample['output']
                     else:
                         inputval = outputval = sample
 
@@ -744,10 +781,6 @@ class TestCodecs(tb.ConnectedTestCase):
     async def test_range_types(self):
         """Test encoding/decoding of range types."""
 
-        if self.server_version < (9, 2):
-            raise unittest.SkipTest(
-                'PostgreSQL servers < 9.2 do not support range types.')
-
         cases = [
             ('int4range', [
                 [(1, 9), asyncpg.Range(1, 10)],
@@ -1096,30 +1129,6 @@ class TestCodecs(tb.ConnectedTestCase):
         finally:
             await conn.close()
 
-    async def test_custom_codec_override_deprecation(self):
-        conn = await self.cluster.connect(database='postgres', loop=self.loop)
-        try:
-            def _encoder(value):
-                return value
-
-            def _decoder(value):
-                return value
-
-            with self.assertWarnsRegex(DeprecationWarning,
-                                       r"The `binary` keyword argument to "
-                                       r"set_type_codec\(\) is deprecated"):
-                await conn.set_type_codec(
-                    'uuid', encoder=_encoder, decoder=_decoder,
-                    schema='pg_catalog', binary=False
-                )
-
-                data = '14058ad9-0118-4b7e-ac15-01bc13e2ccd1'
-                res = await conn.fetchval('SELECT $1::uuid', data)
-                self.assertEqual(res, data)
-
-        finally:
-            await conn.close()
-
     async def test_composites_in_arrays(self):
         await self.con.execute('''
             CREATE TYPE t AS (a text, b int);
@@ -1402,3 +1411,7 @@ class TestCodecs(tb.ConnectedTestCase):
                 DROP TABLE testtab;
                 DROP TYPE enum_t;
             ''')
+
+    async def test_no_result(self):
+        st = await self.con.prepare('rollback')
+        self.assertTupleEqual(st.get_attributes(), ())
