@@ -10,6 +10,7 @@ import datetime
 import io
 import tempfile
 
+import asyncpg
 from asyncpg import _testbase as tb
 from asyncpg import compat
 
@@ -217,7 +218,7 @@ class TestCopyFrom(tb.ConnectedTestCase):
                 )
 
     async def test_copy_from_query_to_bad_output(self):
-        with self.assertRaisesRegexp(TypeError, 'output is expected to be'):
+        with self.assertRaisesRegex(TypeError, 'output is expected to be'):
             await self.con.copy_from_query('''
                 SELECT
                     i, i * 10
@@ -418,12 +419,19 @@ class TestCopyTo(tb.ConnectedTestCase):
             )
             f.seek(0)
 
+            if self.con.get_server_version() < (9, 4):
+                force_null = None
+                forced_null_expected = 'n-u-l-l'
+            else:
+                force_null = ('b~',)
+                forced_null_expected = None
+
             res = await self.con.copy_to_table(
                 'copytab', source=f, columns=('a', 'b~'),
                 schema_name='public', format='csv',
                 delimiter='|', null='n-u-l-l', header=True,
                 quote='*', escape='!', force_not_null=('a',),
-                force_null=('b~',))
+                force_null=force_null)
 
             self.assertEqual(res, 'COPY 7')
 
@@ -435,7 +443,7 @@ class TestCopyTo(tb.ConnectedTestCase):
             self.assertEqual(
                 output,
                 [
-                    ('*', None, None),
+                    ('*', forced_null_expected, None),
                     ('a1', 'b1', None),
                     ('a2', 'b2', None),
                     ('a3', 'b3', None),
@@ -505,7 +513,7 @@ class TestCopyTo(tb.ConnectedTestCase):
                 async def __anext__(self):
                     raise RuntimeError('failure in source')
 
-            with self.assertRaisesRegexp(RuntimeError, 'failure in source'):
+            with self.assertRaisesRegex(RuntimeError, 'failure in source'):
                 await self.con.copy_to_table('copytab', source=_Source())
 
             # Check that the protocol has recovered.
@@ -535,7 +543,7 @@ class TestCopyTo(tb.ConnectedTestCase):
                     else:
                         raise RuntimeError('failure in source')
 
-            with self.assertRaisesRegexp(RuntimeError, 'failure in source'):
+            with self.assertRaisesRegex(RuntimeError, 'failure in source'):
                 await self.con.copy_to_table('copytab', source=_Source())
 
             # Check that the protocol has recovered.
@@ -618,7 +626,7 @@ class TestCopyTo(tb.ConnectedTestCase):
             records = [('2975ab9a-f79c-4ab4-9be5-7bc134d952f0',)]
 
             with self.assertRaisesRegex(
-                    RuntimeError, 'no binary format encoder'):
+                    asyncpg.InternalClientError, 'no binary format encoder'):
                 await self.con.copy_records_to_table(
                     'copytab', records=records)
 
