@@ -193,6 +193,11 @@ class TestPool(tb.ConnectedTestCase):
         async with pool.acquire() as con:
             self.assertIn(repr(con._con), repr(con))  # Test __repr__.
 
+            ps = await con.prepare('SELECT 1')
+            async with con.transaction():
+                cur = await con.cursor('SELECT 1')
+                ps_cur = await ps.cursor()
+
         self.assertIn('[released]', repr(con))
 
         with self.assertRaisesRegex(
@@ -201,6 +206,32 @@ class TestPool(tb.ConnectedTestCase):
 
             con.execute('select 1')
 
+        for meth in ('fetchval', 'fetchrow', 'fetch', 'explain',
+                     'get_query', 'get_statusmsg', 'get_parameters',
+                     'get_attributes'):
+            with self.assertRaisesRegex(
+                    asyncpg.InterfaceError,
+                    r'cannot call PreparedStatement\.{meth}.*released '
+                    r'back to the pool'.format(meth=meth)):
+
+                getattr(ps, meth)()
+
+        for c in (cur, ps_cur):
+            for meth in ('fetch', 'fetchrow'):
+                with self.assertRaisesRegex(
+                        asyncpg.InterfaceError,
+                        r'cannot call Cursor\.{meth}.*released '
+                        r'back to the pool'.format(meth=meth)):
+
+                    getattr(c, meth)()
+
+            with self.assertRaisesRegex(
+                    asyncpg.InterfaceError,
+                    r'cannot call Cursor\.forward.*released '
+                    r'back to the pool'.format(meth=meth)):
+
+                c.forward(1)
+
         await pool.close()
 
     async def test_pool_12(self):
@@ -227,6 +258,11 @@ class TestPool(tb.ConnectedTestCase):
 
         await pool.close()
 
+    def test_pool_init_run_until_complete(self):
+        pool_init = self.create_pool(database='postgres')
+        pool = self.loop.run_until_complete(pool_init)
+        self.assertIsInstance(pool, asyncpg.pool.Pool)
+
     async def test_pool_exception_in_setup_and_init(self):
         class Error(Exception):
             pass
@@ -581,9 +617,30 @@ class TestPool(tb.ConnectedTestCase):
 
         self.assertGreaterEqual(N, 50)
 
+    async def test_pool_handles_inactive_connection_errors(self):
+        pool = await self.create_pool(database='postgres',
+                                      min_size=1, max_size=1)
+
+        con = await pool.acquire(timeout=POOL_NOMINAL_TIMEOUT)
+
+        true_con = con._con
+
+        await pool.release(con)
+
+        # we simulate network error by terminating the connection
+        true_con.terminate()
+
+        # now pool should reopen terminated connection
+        con = await pool.acquire(timeout=POOL_NOMINAL_TIMEOUT)
+
+        self.assertEqual(await con.fetchval('SELECT 1'), 1)
+
+        await con.close()
+        await pool.close()
+
 
 @unittest.skipIf(os.environ.get('PGHOST'), 'using remote cluster for testing')
-class TestHostStandby(tb.ConnectedTestCase):
+class TestHotStandby(tb.ConnectedTestCase):
     @classmethod
     def setUpClass(cls):
         super().setUpClass()
@@ -650,3 +707,14 @@ class TestHostStandby(tb.ConnectedTestCase):
                 tasks = [worker() for _ in range(n)]
                 await asyncio.gather(*tasks, loop=self.loop)
                 await pool.close()
+
+    async def test_standby_cursors(self):
+        con = await self.standby_cluster.connect(
+            database='postgres', loop=self.loop)
+
+        try:
+            async with con.transaction():
+                cursor = await con.cursor('SELECT 1')
+                self.assertEqual(await cursor.fetchrow(), (1,))
+        finally:
+            await con.close()
